==> bigloo1.3b.tar.gz: bigloo1.3/examples/Lower/lower.scm <==
;*---------------------------------------------------------------------*/
;*    serrano/prgm/project/bigloo/examples/Lower/lower.scm ...         */
;*                                                                     */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Mon Oct 19 14:49:54 1992                          */
;*    Last change :  Mon May 10 08:26:33 1993  (serrano)               */
;*                                                                     */
;*    Un petit programme qui prends un fichier `Lisp' et qui           */
;*    convertit tous les symboles en minuscule.                        */
;*---------------------------------------------------------------------*/

;*---------------------------------------------------------------------*/
;*    Le module                                                        */
;*---------------------------------------------------------------------*/
(MODULE LOWER
   (MAIN MAIN))

;*---------------------------------------------------------------------*/
;*    main ...                                                         */
;*---------------------------------------------------------------------*/
(DEFINE (MAIN ARGV)
   (IF (NULL? (CDR ARGV))
       (BEGIN
	  (PRINT "usage: lower [file1] ... [filen]")
	  (exit -1))
       (FOR-EACH LOWERFILE (CDR ARGV))))

;*---------------------------------------------------------------------*/
;*    lowerfile ...                                                    */
;*---------------------------------------------------------------------*/
(DEFINE (LOWERFILE FILENAME)
   (let ((input-port  (open-INPUT-file filename))
	 (output-port (open-output-file (string-append filename "l"))))
      (cond
	 ((not (input-port? input-port))
	  (error "lower" "can't open input file" filename))
	 ((NOT (output-port? output-port))
	  (error "lower"
		 "can't open output file"
		 (string-append filename "l")))
	 (else
	  (lower input-port output-port)))))

;*---------------------------------------------------------------------*/
;*    lower ...                                                        */
;*---------------------------------------------------------------------*/
(define lower
   (let* ((par-open 0)
	  (grammar (regular-grammar ((chiffre (>-< #\0 #\9))
				     (float   (! ((* chiffre) #\. (+ chiffre))
						 ((+ chiffre) #\. (* chiffre))))
				     (lettre  (>-< #\a #\z #\A #\Z))
				     (special (in #\! #\@ #\# #\$ #\%
						  #\^ #\& #\* #\> #\<
						  #\/ #\. #\- #\_ #\+   
						  #\\ #\| #\{ #\= #\?
						  #\} #\: #\~))
				     (quote   (in #\" #\, #\' #\`))
				     (paren   (in #\( #\) #\[ #\]))
				     (blank   (in #\space #\tab #\newline))  
				     (id      (+  (! lettre chiffre special))))
		      ((blank)
		       (the-string))
		      ((#\# #\\ (! lettre
				   chiffre
				   special
				   quote
				   paren
				   ";"))
		       (the-string))
		      ((";" (* (all))) 
		       (the-string))
		      ((uncase "#\newline")
		       (the-string))
		      ((uncase "#\tab")
		       (the-string))
		      ((uncase "#\space")
		       (the-string))
		      ((#\" (* (! (<-> #\")
				  (#\\ #\"))) #\")
		       (the-string))
		      ((eof (#\" (* (! (<-> #\")
				       (#\\ #\")))))
		       (error "read" "Unexpected end-of-file" 'string))
		      ((! (+ chiffre)
			  (#\- (+ chiffre))
			  (#\+ (+ chiffre)))
		       (the-string))
		      (("#o" (! (+ (>-< #\0 #\7))
				((in #\+ #\-)
				 (>-< #\0 #\7))))
		       (the-string))
		      (("#d" (! (+ chiffre)
				((in #\+ #\-)
				 chiffre)))
		       (the-string))
		      (("#x" (! (+ (! chiffre
				      (>-< #\a #\f)
				      (>-< #\A #\F)))
				((in #\+ #\-)
				 (+ (! chiffre
				       (>-< #\a #\f)
				       (>-< #\A #\F))))))
		       (the-string))
		      ((! float
			  ((in #\+ #\-) float)
			  ((! float (+ chiffre))
			   (in #\e #\E) (+ chiffre)) 
			  ((in #\+ #\-) (! float (+ chiffre))
					(in #\e #\E) (+ chiffre))
			  ((! float (+ chiffre)) (in #\e #\E)
						 (in #\+ #\-) (+ chiffre))
			  ((in #\+ #\-) (! float (+ chiffre))
					(in #\e #\E) (in #\+ #\-) (+ chiffre)))
		       (the-string))
		      (pair (#\.)
			    (context)
			    'dot)
		      ((uncase "#t")
		       (the-string))
		      ((uncase "#f")
		       (the-string))
		      ((! id (#\. (+ #\.)))
		       (string-downcase (the-string)))
		      ((#\')
		       (cons 'quote (cons (ignore) '())))
		      ((#\`)
		       (cons 'quasiquote (cons (ignore) '())))
		      ((#\,)
		       (cons 'unquote (cons (ignore) '())))
		      ((#\, #\@)
		       (cons 'unquote-splicing (cons (ignore) '())))
		      ((in #\( #\[)
		       (let ((open-key par-open))
			  (set! par-open (+ 1 par-open))
			  (context 'pair)
			  (let loop-pair ((walk (ignore)))
			     (cond
				((= open-key par-open)
				 '())
				((eq? walk 'dot)
				 (let ((cdr (ignore)))
				    (ignore)
				    (if (= open-key par-open)
					cdr
					(begin
					   (error "read" "illegal pair" cdr)
					   (let loop ()
					      (if (= par-open open-key)
						  (ignore)
						  (begin
						     (ignore)
						     (loop))))))))
				(else
				 (cons walk (loop-pair (ignore))))))))
		      ((in #\) #\])
		       (set! par-open (- par-open 1))
		       (if (< par-open 0)
			   (begin
			      (set! par-open 0)
			      (ignore))
			   #f))
		      ((#\# #\()
		       (let ((car      (ignore)) 
			     (open-key par-open))
			  (set! par-open (+ 1 par-open))
			  (list->vector
			   (let loop-vector ((walk car))
			      (cond
				 ((= open-key par-open)
				  '())
				 (else
				  (cons walk
					(loop-vector (ignore)))))))))
		      (else
		       (let ((char (the-failing-char)))
			  (if (and (eof-object? char) (> par-open 0))
			      (error "read" "Unexpected end-of-file" char)
			      char))))))
      (lambda (input-port output-port)
	 (let loop ((sexp (read/rp grammar input-port)))
	    (if (eof-object? sexp)
		'done
		(begin
		   (display-sexp sexp output-port)
		   (loop (read/rp grammar input-port)))))))) 

;*---------------------------------------------------------------------*/
;*    display-sexp ...                                                 */
;*---------------------------------------------------------------------*/
(define (display-sexp sexp port)
   (cond
      ((pair? sexp)
       (cond
	  ((eq? (car sexp) 'quote)
	   (display "'" port)
	   (display (cadr sexp) port))
	  ((eq? (car sexp) 'unquote)
	   (display "," port)
	   (display (cadr sexp) port))
	  ((eq? (car sexp) 'quasiquote)
	   (display "`" port)
	   (display (cadr sexp) port))
	  ((eq? (car sexp) 'unquote-splicing)
	   (display ",@" port)
	   (display (cadr sexp) port))
	  (else
	   (let ((l sexp))
	      (display "(" port)
	      (for-each (lambda (exp) (display-sexp exp port))
			sexp)
	      (display ")" port)))))
      ((vector? sexp)
       (display "#(" port)
       (let loop ((i 0))
	  (if (= i (vector-length sexp))
	      (display "#)" port)
	      (begin
		 (display (vector-ref sexp i) port)
		 (loop (+ i 1))))))
      (else
       (display sexp port))))

==> bigloo2.0a.tar.gz: bigloo2.0a/examples/Lower/lower.scm <==
;*---------------------------------------------------------------------*/
;*   A pratical implementation for the Scheme programming language     */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*   Copyright (c) 1992-1999 Manuel Serrano                            */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo-request@kaolin.unice.fr                                */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*---------------------------------------------------------------------*/
;*    serrano/prgm/project/bigloo/examples/Lower/lower.scm             */
;*                                                                     */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Mon Oct 19 14:49:54 1992                          */
;*    Last change :  Mon Jan 18 20:56:49 1999 (serrano)                */
;*                                                                     */
;*    Un petit programme qui prends un fichier `Lisp' et qui           */
;*    convertit tous les symboles en minuscule.                        */
;*---------------------------------------------------------------------*/

;*---------------------------------------------------------------------*/
;*    Le module                                                        */
;*---------------------------------------------------------------------*/
(MODULE LOWER
   (MAIN MAIN))

;*---------------------------------------------------------------------*/
;*    main ...                                                         */
;*---------------------------------------------------------------------*/
(DEFINE (MAIN ARGV)
   (IF (NULL? (CDR ARGV))
       (BEGIN
	  (PRINT "usage: lower [file1] ... [filen]")
	  (exit -1))
       (FOR-EACH LOWERFILE (CDR ARGV))))

;*---------------------------------------------------------------------*/
;*    lowerfile ...                                                    */
;*---------------------------------------------------------------------*/
(DEFINE (LOWERFILE FILENAME)
   (let ((input-port  (open-INPUT-file filename))
	 (output-port (open-output-file (string-append filename "l"))))
      (cond
	 ((not (input-port? input-port))
	  (error "lower" "can't open input file" filename))
	 ((NOT (output-port? output-port))
	  (error "lower"
		 "can't open output file"
		 (string-append filename "l")))
	 (else
	  (lower input-port output-port)))))

;*---------------------------------------------------------------------*/
;*    lower ...                                                        */
;*---------------------------------------------------------------------*/
(define lower
   (let* ((par-open 0)
	  (grammar (regular-grammar ((chiffre (in ("09")))
				     (float   (or (: (* chiffre) #\. (+ chiffre))
						  (: (+ chiffre) #\. (* chiffre))))
	 			     (lettre  (in ("azAZ")))
				     (special (in #\! #\@ #\# #\$ #\%
						  #\^ #\& #\* #\> #\<
						  #\/ #\. #\- #\_ #\+   
						  #\\ #\| #\{ #\= #\?
						  #\} #\: #\~))
				     (quote   (in #\" #\, #\' #\`))
				     (paren   (in #\( #\) #\[ #\]))
				     (blank   (in #\space #\tab #\newline))  
				     (id      (+  (or lettre chiffre special))))
		      (blank
		       (the-string))
		      ((: #\# #\\ (or lettre
				      chiffre
				      special
				      quote
				      paren
				      ";"))
		       (the-string))
		      ((: ";" (* all)) 
		       (the-string))
		      ((uncase "#\newline")
		       (the-string))
		      ((uncase "#\tab")
		       (the-string))
		      ((uncase "#\space")
		       (the-string))
		      ((: #\" (* (or (out #\") (: #\\ #\"))) #\")
		       (the-string))
		      ((eof (: #\" (* (or (out #\") (: #\\ #\")))))
		       (error "read" "Unexpected end-of-file" 'string))
		      ((or (+ chiffre)
			   (: #\- (+ chiffre))
			   (: #\+ (+ chiffre)))
		       (the-string))
		      ((: "#o" (or (+ (in ("07")))
				   (: (in #\+ #\-)
				      (+ (in ("07"))))))
		       (the-string))
		      ((: "#d" (or (+ chiffre)
				   (: (in #\+ #\-) (+ chiffre))))
		       (the-string))
		      ((: "#x" (or (+ (or chiffre
					  (uncase (in "af"))))
				   (: (in #\+ #\-)
				      (+ (or chiffre
					     (uncase (in "af")))))))
		       (the-string))
		      ((or float
			   (: (in #\+ #\-) float)
			   (: (or float (+ chiffre))
			      (in #\e #\E) (+ chiffre)) 
			   (: (in #\+ #\-) (or float (+ chiffre))
					   (in #\e #\E) (+ chiffre))
			   (: (or float (+ chiffre)) (in #\e #\E)
						   (in #\+ #\-) (+ chiffre))
			   (: (in #\+ #\-) (or float (+ chiffre))
					   (in #\e #\E)
					   (in #\+ #\-)
					   (+ chiffre)))
		       (the-string))
		      ((context (pair) #\.)
		       (rgc-context)
		       'dot)
		      ((uncase "#t")
		       (the-string))
		      ((uncase "#f")
		       (the-string))
		      ((or id (: #\. (+ #\.)))
		       (string-downcase (the-string)))
		      (#\'
		       (cons 'quote (cons (ignore) '())))
		      (#\`
		       (cons 'quasiquote (cons (ignore) '())))
		      (#\,
		       (cons 'unquote (cons (ignore) '())))
		      ((: #\, #\@)
		       (cons 'unquote-splicing (cons (ignore) '())))
		      ((in #\( #\[)
		       (let ((open-key par-open))
			  (set! par-open (+ 1 par-open))
			  (rgc-context 'pair)
			  (let loop-pair ((walk (ignore)))
			     (cond
				((= open-key par-open)
				 '())
				((eq? walk 'dot)
				 (let ((cdr (ignore)))
				    (ignore)
				    (if (= open-key par-open)
					cdr
					(begin
					   (error "read" "illegal pair" cdr)
					   (let loop ()
					      (if (= par-open open-key)
						  (ignore)
						  (begin
						     (ignore)
						     (loop))))))))
				(else
				 (cons walk (loop-pair (ignore))))))))
		      ((in #\) #\])
		       (set! par-open (- par-open 1))
		       (if (< par-open 0)
			   (begin
			      (set! par-open 0)
			      (ignore))
			   #f))
		      ((: #\# #\()
		       (let ((car      (ignore)) 
			     (open-key par-open))
			  (set! par-open (+ 1 par-open))
			  (list->vector
			   (let loop-vector ((walk car))
			      (cond
				 ((= open-key par-open)
				  '())
				 (else
				  (cons walk
					(loop-vector (ignore)))))))))
		      (else
		       (let ((char (the-failure)))
			  (if (and (eof-object? char) (> par-open 0))
			      (error "read" "Unexpected end-of-file" char)
			      char))))))
      (lambda (input-port output-port)
	 (let loop ((sexp (read/rp grammar input-port)))
	    (if (eof-object? sexp)
		'done
		(begin
		   (display-sexp sexp output-port)
		   (loop (read/rp grammar input-port)))))))) 

;*---------------------------------------------------------------------*/
;*    display-sexp ...                                                 */
;*---------------------------------------------------------------------*/
(define (display-sexp sexp port)
   (cond
      ((pair? sexp)
       (cond
	  ((eq? (car sexp) 'quote)
	   (display "'" port)
	   (display (cadr sexp) port))
	  ((eq? (car sexp) 'unquote)
	   (display "," port)
	   (display (cadr sexp) port))
	  ((eq? (car sexp) 'quasiquote)
	   (display "`" port)
	   (display (cadr sexp) port))
	  ((eq? (car sexp) 'unquote-splicing)
	   (display ",@" port)
	   (display (cadr sexp) port))
	  (else
	   (let ((l sexp))
	      (display "(" port)
	      (for-each (lambda (exp) (display-sexp exp port))
			sexp)
	      (display ")" port)))))
      ((vector? sexp)
       (display "#(" port)
       (let loop ((i 0))
	  (if (= i (vector-length sexp))
	      (display "#)" port)
	      (begin
		 (display (vector-ref sexp i) port)
		 (loop (+ i 1))))))
      (else
       (display sexp port))))
